@@ -57,6 +57,10 @@ struct __fehoptions {
 #ifdef HAVE_LIBEXIF
 	unsigned char draw_exif;
 	unsigned char auto_rotate;
+#endif
+#ifdef HAVE_INOTIFY
+	unsigned char auto_reload;
+    int inotify_fd;
 #endif
 	unsigned char list;
 	unsigned char quiet;
